<?php

class Cashtronics extends PaymentModule
{
	private	$_html = '';
	private $_postErrors = array();

	public function __construct()
	{
		$this->name = 'cashtronics';
		$this->tab = 'Payment';
		$this->version = '1.0';
		
		$this->currencies = true;
		$this->currencies_mode = 'radio';

        parent::__construct();

        /* The parent construct is required for translations */
		$this->page = basename(__FILE__, '.php');
        $this->displayName = $this->l('PayPal');
        $this->description = $this->l('Accepts payments by Cashtronics');
		$this->confirmUninstall = $this->l('Are you sure you want to delete your details ?');
	}

	public function getPaypalUrl()
	{
			return 'https://esecure2.cashtronics.eu/epxsecure/ctronics-cb.php';
	}

	public function install()
	{
		if (!parent::install() OR !Configuration::updateValue('CASHTRONICS_MERCHANT_NAME', 'your_merchant_name')
			OR !Configuration::updateValue('CASHTRONICS_EMAIL', 'you@yourdomain.com') 
			OR !Configuration::updateValue('CASHTRONICS_MERCHANT_CODE', 'your_merchant_code')
			OR !Configuration::updateValue('CASHTRONICS_MERCHANT_KEY', 'your_merchant_key')
			OR !Configuration::updateValue('CASHTRONICS_TERMINAL', 'terminal number')
			OR !$this->registerHook('payment'))
			return false;
		return true;
	}

	public function uninstall()
	{
		if (!Configuration::deleteByName('CASHTRONICS_MERCHANT_NAME') OR !Configuration::deleteByName('CASHTRONICS_EMAIL')
		OR !Configuration::deleteByName('CASHTRONICS_MERCHANT_CODE')
		OR !Configuration::deleteByName('CASHTRONICS_MERCHANT_KEY')
		OR !Configuration::deleteByName('CASHTRONICS_TERMINAL')
			OR !parent::uninstall())
			return false;
		return true;
	}

	public function getContent()
	{
		$this->_html = '<h2>Cashtronics</h2>';
		if (isset($_POST['submitCashtronics']))
		{
			if (empty($_POST['merchant_name']))
				$this->_postErrors[] = $this->l('Cashtronics Merchant Name - this is required');
				
			if (empty($_POST['merchant_code']))
				$this->_postErrors[] = $this->l('Cashtronics Merchant Code- this is required');

			if (empty($_POST['merchant_key']))
				$this->_postErrors[] = $this->l('Cashtronics Merchant Key - this is required');
				
			if (empty($_POST['terminal']))
				$this->_postErrors[] = $this->l('Cashtronics Terminal- this is required');
				
			if (empty($_POST['email']))
				$this->_postErrors[] = $this->l('Cashtronics Email Address - this is required');
			elseif (!Validate::isEmail($_POST['email']))
				$this->_postErrors[] = $this->l('Cashtronics Email Address - must be an e-mail address.');
				
			if (!sizeof($this->_postErrors))
			{
				Configuration::updateValue('CASHTRONICS_MERCHANT_NAME', $_POST['merchant_name']);
				Configuration::updateValue('CASHTRONICS_EMAIL', intval($_POST['email']));
				Configuration::updateValue('CASHTRONICS_MERCHANT_CODE', intval($_POST['merchant_code']));
				Configuration::updateValue('CASHTRONICS_MERCHANT_KEY', intval($_POST['merchant_key']));
				Configuration::updateValue('CASHTRONICS_TERMINAL', intval($_POST['terminal']));

				$this->displayConf();
			}
			else
				$this->displayErrors();
		}

		$this->displayCashtronics();
		$this->displayFormSettings();
		return $this->_html;
	}

	public function displayConf()
	{
		$this->_html .= '
		<div class="conf confirm">
			<img src="../img/admin/ok.gif" alt="'.$this->l('Confirmation').'" />
			'.$this->l('Settings updated').'
		</div>';
	}

	public function displayErrors()
	{
		$nbErrors = sizeof($this->_postErrors);
		$this->_html .= '
		<div class="alert error">
			<h3>'.($nbErrors > 1 ? $this->l('There are') : $this->l('There is')).' '.$nbErrors.' '.($nbErrors > 1 ? $this->l('errors') : $this->l('error')).'</h3>
			<ol>';
		foreach ($this->_postErrors AS $error)
			$this->_html .= '<li>'.$error.'</li>';
		$this->_html .= '
			</ol>
		</div>';
	}
	
	
	public function displayPayPal()
	{
		$this->_html .= '
		<img src="../modules/paypal/paypal.gif" style="float:left; margin-right:15px;" />
		<b>'.$this->l('This module allows you to accept payments by Cashtronics.').'</b><br /><br />
		'.$this->l('If the client chooses this payment mode, your Cashtronics account will be automatically credited.').'<br />
		'.$this->l('You need to enter your Cashtronics account details first before using this module.').'
		<br /><br /><br />';
	}

	public function displayFormSettings()
	{
	
	
		$conf = Configuration::getMultiple(array('CASHTRONICS_MERCHANT_NAME', 'CASHTRONICS_EMAIL', 'CASHTRONICS_MERCHANT_CODE', 'CASHTRONICS_MERCHANT_KEY', 'CASHTRONICS_TERMINAL'));
		$merchant_name = array_key_exists('merchant_name', $_POST) ? $_POST['merchant_name'] : (array_key_exists('CASHTRONICS_MERCHANT_NAME', $conf) ? $conf['CASHTRONICS_MERCHANT_NAME'] : '');		        $email = array_key_exists('email', $_POST) ? $_POST['email'] : (array_key_exists('CASHTRONICS_EMAIL', $conf) ? $conf['CASHTRONICS_EMAIL'] : '');
		$merchant_code = array_key_exists('merchant_code', $_POST) ? $_POST['merchant_code'] : (array_key_exists('CASHTRONICS_MERCHANT_CODE', $conf) ? $
		$merchant_key = array_key_exists('merchant_key', $_POST) ? $_POST['merchant_key'] : (array_key_exists('CASHTRONICS_MERCHANT_KEY', $conf) ? $
		$terminal = array_key_exists('terminal', $_POST) ? $_POST['terminal'] : (array_key_exists('CASHTRONICS_TERMINAL', $conf) ? $
		
		$this->_html .= '
		<form action="'.$_SERVER['REQUEST_URI'].'" method="post">
		<fieldset>
			<legend><img src="../img/admin/contact.gif" />'.$this->l('Settings').'</legend>
			<label>'.$this->l('Cashtronics Merchant Name').'</label>
			<div class="margin-form"><input type="text" size="33" name="merchant_name" value="'.htmlentities($merchant_name, ENT_COMPAT, 'UTF-8').'" /></div>
			<label>'.$this->l('Cashtronics Email').'</label>
			<div class="margin-form"><input type="text" size="33" name="email" value="'.htmlentities($email, ENT_COMPAT, 'UTF-8').'" /></div>
			<label>'.$this->l('Cashtronics Merchant Code').'</label>
			<div class="margin-form"><input type="text" size="33" name="merchant_code" value="'.htmlentities($merchant_code, ENT_COMPAT, 'UTF-8').'" /></div>
			<label>'.$this->l('Cashtronics Merchant Key').'</label>
			<div class="margin-form"><input type="text" size="33" name="merchant_key" value="'.htmlentities($merchant_key, ENT_COMPAT, 'UTF-8').'" /></div>
			<label>'.$this->l('Cashtronics Terminal').'</label>
			<div class="margin-form"><input type="text" size="33" name="terminal" value="'.htmlentities($terminal, ENT_COMPAT, 'UTF-8').'" /></div>						
			<br /><center><input type="submit" name="submitPaypal" value="'.$this->l('Update settings').'" class="button" /></center>
		</fieldset>
		</form><br /><br />
	}

	public function hookPayment($params)
	{
		global $smarty;

		$address = new Address(intval($params['cart']->id_address_invoice));
		$customer = new Customer(intval($params['cart']->id_customer));
		$business = Configuration::get('PAYPAL_BUSINESS');
		$currency = $this->getCurrency();

		if (!Validate::isEmail($business))
			return $this->l('Paypal error: (invalid or undefined business account email)');

		if (!Validate::isLoadedObject($address) OR !Validate::isLoadedObject($customer) OR !Validate::isLoadedObject($currency))
			return $this->l('Paypal error: (invalid address or customer)');
			
		$products = $params['cart']->getProducts();
		
		foreach ($products as $key => $product)
		{
			$products[$key]['name'] = str_replace('"', '\'', $product['name']);
			if (isset($product['attributes']))
				$products[$key]['attributes'] = str_replace('"', '\'', $product['attributes']);
			$products[$key]['name'] = htmlentities(utf8_decode($product['name']));
			$products[$key]['paypalAmount'] = number_format(Tools::convertPrice($product['price_wt'], $currency), 2, '.', '');
		}
		$smarty->assign(array(
			'address' => $address,
			'country' => new Country(intval($address->id_country)),
			'customer' => $customer,
			'business' => $business,
			'currency' => $currency,
			'paypalUrl' => $this->getPaypalUrl(),
			'amount' => number_format(Tools::convertPrice($params['cart']->getOrderTotal(true, 4), $currency), 2, '.', ''),
			'shipping' =>  number_format(Tools::convertPrice(($params['cart']->getOrderShippingCost() + $params['cart']->getOrderTotal(true, 6)), $currency), 2, '.', ''),
			'discounts' => $params['cart']->getDiscounts(),
			'products' => $products,
			'total' => number_format(Tools::convertPrice($params['cart']->getOrderTotal(true, 3), $currency), 2, '.', ''),
			'id_cart' => intval($params['cart']->id),
			'goBackUrl' => 'http://'.htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8').__PS_BASE_URI__.'order-confirmation.php?key='.$customer->secure_key.'&id_cart='.intval($params['cart']->id).'&id_module='.intval($this->id),
			'returnUrl' => 'http://'.htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8').__PS_BASE_URI__.'modules/paypal/validation.php',
			'this_path' => $this->_path
		));

		return $this->display(__FILE__, 'paypal.tpl');
    }
	
	public function getL($key)
	{
		$translations = array(
			'mc_gross' => $this->l('Paypal key \'mc_gross\' not specified, can\'t control amount paid.'),
			'payment_status' => $this->l('Paypal key \'payment_status\' not specified, can\'t control payment validity'),
			'payment' => $this->l('Payment: '),
			'custom' => $this->l('Paypal key \'custom\' not specified, can\'t rely to cart'),
			'txn_id' => $this->l('Paypal key \'txn_id\' not specified, transaction unknown'),
			'mc_currency' => $this->l('Paypal key \'mc_currency\' not specified, currency unknown'),
			'cart' => $this->l('Cart not found'),
			'order' => $this->l('Order has already been placed'),
			'transaction' => $this->l('Paypal Transaction ID: '),
			'verified' => $this->l('The PayPal transaction could not be VERIFIED.'),
			'connect' => $this->l('Problem connecting to the PayPal server.'),
			'nomethod' => $this->l('No communications transport available.'),
			'socketmethod' => $this->l('Verification failure (using fsockopen). Returned: '),
			'curlmethod' => $this->l('Verification failure (using cURL). Returned: '),
			'curlmethodfailed' => $this->l('Connection using cURL failed'),
		);
		return $translations[$key];
	}
}

?>
